#!/usr/bin/env python3

# Copyright 2014 Brett Slatkin, Pearson Education Inc.
#
# Udostępniono na licencji Apache w wersji 2.0 ("Licencja").
# Tego pliku można używać jedynie zgodnie z warunkami Licencji.
# Treść Licencji znajdziesz na stronie:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# O ile obowiązujące prawo nie stanowi inaczej lub czegoś innego nie
# uzgodniono w formie pisemnej, oprogramowanie objęte Licencją jest
# dostarczane w stanie, w jakim jest (wersja "AS IS"), BEZ JAKIEJKOLWIEK
# GWARANCJI, ani wyrażonej otwarcie, ani domyślnej. Dokładne zasady
# i warunki Licencji znajdziesz w jej treści.

# Przygotowania mające na celu odtworzenie środowiska użytego w książce.
import logging
from pprint import pprint
from sys import stdout as STDOUT


# Przykład 1.
def index_words(text):
    result = []
    if text:
        result.append(0)
    for index, letter in enumerate(text):
        if letter == ' ':
            result.append(index + 1)
    return result


# Przykład 2.
address = 'Lat temu osiemdziesiąt i siedem...'
address = 'Lat temu osiemdziesiąt i siedem nasi ojcowie wznieśli na tym kontynencie nowy naród, poczęty z wolności i oddany w przekonaniu, że wszyscy ludzie zostali stworzeni równymi.'
result = index_words(address)
print(result[:3])


# Przykład 3.
def index_words_iter(text):
    if text:
        yield 0
    for index, letter in enumerate(text):
        if letter == ' ':
            yield index + 1


# Przykład 4.
result = list(index_words_iter(address))
print(result[:3])


# Przykład 5.
def index_file(handle):
    offset = 0
    for line in handle:
        if line:
            yield offset
        for letter in line:
            offset += 1
            if letter == ' ':
                yield offset


# Przykład 6.
address_lines = """Lat temu osiemdziesiąt i siedem
nasi ojcowie wznieśli na tym kontynencie nowy naród,
poczęty z wolności i oddany w przekonaniu, że wszyscy
ludzie zostali stworzeni równymi."""

with open('/tmp/address.txt', 'w') as f:
    f.write(address_lines)

from itertools import islice
with open('/tmp/address.txt', 'r') as f:
    it = index_file(f)
    results = islice(it, 0, 3)
    print(list(results))
