#!/usr/bin/env python3

# Copyright 2014 Brett Slatkin, Pearson Education Inc.
#
# Udostępniono na licencji Apache w wersji 2.0 ("Licencja").
# Tego pliku można używać jedynie zgodnie z warunkami Licencji.
# Treść Licencji znajdziesz na stronie:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# O ile obowiązujące prawo nie stanowi inaczej lub czegoś innego nie
# uzgodniono w formie pisemnej, oprogramowanie objęte Licencją jest
# dostarczane w stanie, w jakim jest (wersja "AS IS"), BEZ JAKIEJKOLWIEK
# GWARANCJI, ani wyrażonej otwarcie, ani domyślnej. Dokładne zasady
# i warunki Licencji znajdziesz w jej treści.

# Przygotowania mające na celu odtworzenie środowiska użytego w książce.
import logging
from pprint import pprint
from sys import stdout as STDOUT


# Przykład 1.
class FrequencyList(list):
    def __init__(self, members):
        super().__init__(members)

    def frequency(self):
        counts = {}
        for item in self:
            counts.setdefault(item, 0)
            counts[item] += 1
        return counts


# Przykład 2.
foo = FrequencyList(['a', 'b', 'a', 'c', 'b', 'a', 'd'])
print('Długość wynosi', len(foo))
foo.pop()
print('Po użyciu metody pop():', repr(foo))
print('Częstotliwość:', foo.frequency())


# Przykład 3.
class BinaryNode(object):
    def __init__(self, value, left=None, right=None):
        self.value = value
        self.left = left
        self.right = right


# Przykład 4.
bar = [1, 2, 3]
bar[0]


# Przykład 5.
bar.__getitem__(0)


# Przykład 6.
class IndexableNode(BinaryNode):
    def _search(self, count, index):
        found = None
        if self.left:
            found, count = self.left._search(count, index)
        if not found and count == index:
            found = self
        else:
            count += 1
        if not found and self.right:
            found, count = self.right._search(count, index)
        return found, count
        # Wartością zwrotną jest (found, count).

    def __getitem__(self, index):
        found, _ = self._search(0, index)
        if not found:
            raise IndexError('Wykroczono poza indeks')
        return found.value


# Przykład 7.
tree = IndexableNode(
    10,
    left=IndexableNode(
        5,
        left=IndexableNode(2),
        right=IndexableNode(
            6, right=IndexableNode(7))),
    right=IndexableNode(
        15, left=IndexableNode(11)))


# Przykład 8.
print('LRR =', tree.left.right.right.value)
print('Indeks 0 =', tree[0])
print('Indeks 1 =', tree[1])
print('11 znajduje się w drzewie?', 11 in tree)
print('17 znajduje się w drzewie?', 17 in tree)
print('Drzewo ma postać', list(tree))


# Przykład 9.
try:
    len(tree)
except:
    logging.exception('Oczekiwany')
else:
    assert False


# Przykład 10.
class SequenceNode(IndexableNode):
    def __len__(self):
        _, count = self._search(0, None)
        return count


# Przykład 11.
tree = SequenceNode(
    10,
    left=SequenceNode(
        5,
        left=SequenceNode(2),
        right=SequenceNode(
            6, right=SequenceNode(7))),
    right=SequenceNode(
        15, left=SequenceNode(11))
)

print('Drzewo ma %d węzłów' % len(tree))


# Przykład 12.
try:
    from collections.abc import Sequence
    
    class BadType(Sequence):
        pass
    
    foo = BadType()
except:
    logging.exception('Oczekiwany')
else:
    assert False


# Przykład 13.
class BetterNode(SequenceNode, Sequence):
    pass

tree = BetterNode(
    10,
    left=BetterNode(
        5,
        left=BetterNode(2),
        right=BetterNode(
            6, right=BetterNode(7))),
    right=BetterNode(
        15, left=BetterNode(11))
)

print('Indeks elementu 7 wynosi', tree.index(7))
print('Liczba wystąpień elementu 10 to', tree.count(10))
